(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

(******************************)
(* Test configuration control *)
(******************************)

val get_gcc_opts : Misc.arch -> string
val set_gcc_opts : string -> unit
val default_gcc_opts : Misc.arch -> string

(* Target OS *)
type target_os = Linux | Mac | AIX

(* Use gnu as syntax extensions (namely numeric labels) *)
val get_gas : unit -> bool option
val set_gas : bool -> unit

val get_target_os : Misc.arch -> target_os
val set_os : string -> unit
val pp_os : target_os -> string
val default_os : unit -> string

val get_numeric_labels : Misc.arch -> bool

(* 32/64 mode (impacts on code generation) *)
type word = W32 | W64 | WXX

val get_word :  Misc.arch -> word
val set_word : string -> unit
val default_ws : unit -> string

(* A choice for barriers *)
type barrier = User | Pthread | NoBarrier | User2 | TimeBase | UserFence
val default_barrier : unit -> string
val get_barrier : Misc.arch -> barrier
val set_barrier : string -> unit
val pp_barrier : Misc.arch -> string
(* delay for timebase *)
val default_delay : Misc.arch -> int
val get_delay : Misc.arch -> int
val set_delay :  int -> unit

(* Show time of barrier synchronisation *)
val get_verbose_barrier : unit -> bool
val set_verbose_barrier : bool -> unit
val get_xy : unit -> bool
val set_xy : bool -> unit

(* Contiguous memory: allocate shared locations space as a big chunk *)
val get_contiguous : unit -> bool
val set_contiguous : bool -> unit

(* how threads are started/joined *)
type launch = Fixed | Changing
val default_launch : unit -> string
val get_launch : Misc.arch -> launch
val set_launch : string -> unit
val pp_launch : Misc.arch -> string

(* how memory is accessed *)
type memory = Direct | Indirect
val default_memory : unit -> string
val get_memory : Misc.arch -> memory
val set_memory : string -> unit
val pp_memory : Misc.arch -> string

(* How memory is scanned (stride) *)
val default_stride : unit -> int option
val set_stride : int -> unit
val get_stride : unit -> int option
val pp_stride : int option -> string

(* Safer mode that add a few checks by fear of
   pthread not doing the right thing
   (Have writes to memory 'flushed' before thread creation
    and thread exit.)
     - Check initial state transmission from main thread to
       children.
     - Put fake initial value in variables used to transmit
       register final states
     - Check stabilization  of shared variables.
*)

type safer = NoSafe|AllSafe|WriteSafe

val set_safer : string -> unit
val get_safer : Misc.arch -> safer
val pp_safer :  Misc.arch -> string
val default_safer : unit -> string


(* Preload mode *)
type preload =
  | NoPL (* do nothing *)
  | RandomPL (* load some ramdom subset of shared variables *)
  | CustomPL (* Finer control *)
val set_preload : string -> unit
val pp_preload : preload -> string
val get_preload : unit -> preload
val default_preload : unit -> string

(* How results are collected *)
type collect = CollectLocal|CollectAfter|CollectBoth
val set_collect : string -> unit
val pp_collect : collect -> string
val get_collect : unit -> collect
val default_collect : unit -> string

(* SMT description *)

type smt_mode = SMTSeq | SMTEnd | SMTNone
val set_smt_mode : string -> unit
val pp_smt_mode : smt_mode -> string
val get_smt_mode : unit -> smt_mode
val default_smt_mode : unit -> string

val set_smt : int -> unit
val get_smt : unit -> int
val default_smt : unit -> int



(* Flood avail procs *)
val is_flood : Misc.arch -> bool
val get_avail : Misc.arch -> int option
val set_avail : int -> unit

(* Flood technique *)
type para =
  | Shell (* use shell and Unix processes *)
  | Self  (* run several test instances ourselves [using threads] *)

val set_para : string -> unit
val get_para : unit -> para
val pp_para : unit -> string
val default_para : unit -> string

(* Thread cache *)
val get_cache : unit -> bool
val set_cache : bool -> unit
val default_cache : unit -> bool

(* Call one experiment in place of forking it *)
val get_call : unit -> bool
val set_call : bool -> unit
val default_call : unit -> bool

(* Alloc memory before lauching parallel experiments *)
val get_prealloc : unit -> bool
val set_prealloc : bool -> unit
val default_prealloc : unit -> bool

(* Limit number of procs *)
val get_limit : unit -> bool
val set_limit : bool -> unit

(* Avoid some tests, given by edge cycles *)
val get_no : unit -> string option
val set_no : string -> unit

(* Hints *)
val get_hint : unit -> string option
val set_hint : string -> unit

(* Affinity control *)
type affinity =
  | AffinityNone
  | AffinityIncr of int
  | AffinityRandom
  | AffinityCustom

val get_affinity : unit -> affinity
val set_affinity : string -> unit
val set_affinity_incr : int -> unit
val pp_affinity : affinity -> string
val default_affinity : unit -> string

val set_logical_procs : string -> unit
val get_logical_procs : unit -> int list option

(* Force affinity (useful to shunt power management) *)
val default_force_affinity : bool
val set_force_affinity : bool -> unit
val get_force_affinity : unit -> bool

(* Set size and number of runs *)
val default_size : unit -> int
val default_run : unit -> int
val set_size : int -> unit
val get_size : Misc.arch -> int
val set_run : int -> unit
val get_run : Misc.arch -> int
val dump_bench : Misc.arch -> out_channel -> unit


(* Save compiled sources in index file *)
val set_exp : string -> unit
val get_exp : unit -> string option

(******************)
(* litmus control *)
(******************)

val get_tsc : unit -> bool
val set_tsc : bool -> unit

val get_clocktime : unit -> bool
val set_clocktime : bool -> unit

val get_profiling : unit -> bool
val set_profiling : bool -> unit

val get_time_loop : unit -> int
val set_time_loop : int -> unit

(* isync just before test code *)
val default_isync : unit -> bool
val get_isync : unit -> bool
val set_isync : bool -> unit

(* Sync macro replacement *)
val sync_const : int (* One place for magic constant 128 *)
val get_sync_macro : unit -> int option
val set_sync_macro : int -> unit

(* Signaling write *)
val default_signaling_write : unit -> bool
val get_signaling_write  : unit -> bool
val set_signaling_write  : bool -> unit

(* Suppress final condition quantifier *)
val default_kind : unit -> bool
val get_kind : unit -> bool
val set_kind : bool -> unit

(* Sleep between tests *)
val default_sleep : unit -> int
val get_sleep : unit -> int
val set_sleep : int -> unit

(* gcc name *)
val default_gcc : unit -> string
val get_gcc  : unit -> string
val set_gcc  : string -> unit

(* additional link options *)
val set_link_opt : string -> unit
val get_link_opt : unit -> string

(* Stop test as soon as condition is settled *)
type speedcheck = NoSpeed | SomeSpeed | AllSpeed

val get_speedcheck : unit -> speedcheck
val set_speedcheck : string -> unit
val default_speedcheck : unit -> speedcheck
val pp_speedcheck : speedcheck -> string

(* Run name - identifying string *)
val get_runname : unit -> string
val set_runname : string -> unit

(* Cross compilation *)
val is_out : unit -> bool
val cross_mode : unit -> bool
val set_tar : string -> unit
val set_cross : string -> unit
val get_tar : unit -> string

(* Cross run *)
type addr = { host : string ; port : int option ; }

val default_crossrun : unit -> addr option
val get_crossrun : unit -> addr option
val set_crossrun : string -> unit


